' This script allows you to automate the creation of a 
' VirtualDub "Job Control" (.jobs) file. The created job 
' file will apply the same video and audio processing to 
' groups of files -- in fact, it will apply your filter 
' rules to every video file in a target directory. The 
' starting point for determing what filtering and processing 
' is to be applied is a saved VirtualDub "Configuration" 
' (.vcf) file.
'
' NOTE: VirtualDub should NOT be running if you plan on 
' modifying or creating a jobs file in the VirtualDub 
' program folder. Besides, that folder may need admin 
' priveleges to write into, so there's two reasons why 
' you should stay out of it.
'
' After you have some saved settings, you can start the 
' script in any of a number of ways. You can specify 
' everything on the command line, but nobody does that! It 
' means typing. I hate typing. You can just run the script 
' by double-clicking it. That will ask you for all the 
' data it needs. 
'
' For the file and folder paths the script will require, I 
' regret there is no cool file browser to help you. Cool 
' file browsers worked from Windows 98 through Windows XP, 
' but now we're back to using ugly input boxes.
'
'   VCF file: The full path to the VCF file. 
'
'   Movie extensions: I only work with AVI files, but I 
'   always click OK on the entire list because as long 
'   as I don't have any MPG files in that directory, 
'   it doesn't hurt anything.
'
'   Source: The place you store the source movie files. 
'   If you have an entire tree of folders with movies, 
'   enter the folder at the top of the tree.
'
'   Recursion: Yes or no - Should the script only look 
'   in one folder (No) or search through all folders 
'   below the starting point looking for movies (Yes)?
'
' Now... There are two more questions, but you can't just 
' hit "Cancel" to stop things from this point on. If you 
' do, the script will just interpret the cancel action as 
' permission to proceed with default values. If you really 
' need to stop things, you'll need to use the task manager 
' to kill the "wscript" process (if you're running 
' cscript, you already know how to kill it!).
'
'   Movie destination: If you want all the processed 
'   output movies dumped in one folder, enter the full 
'   path to that folder. That's easy. However, if your 
'   movies are scattered across several different folders, 
'   they might have identical names, meaning the processed 
'   files can't be put in a common location. To solve 
'   this, the script can create new folders to store the 
'   processed movies in. If you want the sub-folders 
'   created, just enter the name to be used. If you don't 
'   enter anything, the base name of the script will be 
'   used as a sub-folder name. For example, if you have 
'   movies in "C:\a\b" and in "C:\a\d", you might tell 
'   the script the movie source folder was "C:\a" and the 
'   destination was just "z". The script would then store 
'   the new movies in "C:\a\b\z" and in "C:\a\d\z". The 
'   script would create the new "z" folders so they'd be 
'   ready for the actual VirtualDub program to use.
'
'   Jobs file destination: Where do you want the created 
'   jobs file to go? You don't get to pick a name (the 
'   name will always match the script name except for the 
'   file extension), but you can pick the folder. If you 
'   don't enter anything here, the jobs file will be 
'   placed in the script's folder.
'
' At this point, you'll have a new jobs file! Open it with 
' Notepad and take a look at it. If the total number of 
' files and the paths look good, you can open VirtualDub, 
' select File / Job Control, then select File / Load Job List. 
'
'----------------------------------------------------------
'
' Command line... I put this last because nobody does it. 
' Just run the script with the /? argument to get help. Any 
' argument you don't supply will result in the interactive 
' nonsense described above to get the missing information. 
' For those values that are allowed to be empty (i.e., movie 
' destination and jobs file destination), you have to have 
' the "switch" there, but it can have an empty value like 
' /Job: (with no value at all) or /Job:"" (with empty quotes). 
'
' The order of the arguments doesn't matter and the argument 
' names are case-insensitive (e.g., /Job: /JOB: and /job: all 
' work the same).
'
' Here's an example command line:
'
' VirtualDub.vbs /VCF:"C:\file.vcf" /Extensions:"avi,mpg" /Source:"C:\OldMovies" /Destination:"NewMovies" /Job:"" /Recurse:True
'
'----------------------------------------------------------
'
' The script and this readme file are both released to the 
' Public Domain by Eric Phelps. Use it, modify it, sell it, 
' and you don't have to get my permission or give me credit. 
' With such generosity on my part comes a total lack of 
' responsibility on my part. The script may not work as 
' described despite my best efforts. The script is, by its 
' nature, "open source", so I require you to assume full 
' responsibility for verifying suitability for your 
' intended use.

' Eric Phelps
' June 2014
' http://www.ericphelps.com/


Option Explicit

'The script uses arguments (like "/Source:") whose names can be changed here:
Const ARG_VCF_FILE = "VCF" 'Name of arg for the input VCF file that has all the settings.
Const ARG_MOVIE_SOURCE_EXTENSIONS = "Extensions" 'Name of arg for the CSV list of movie file extensions.
Const ARG_MOVIE_SOURCE_FOLDER = "Source" 'Name of arg for the starting folder to find movies in. 
Const ARG_MOVIE_DEST_FOLDER = "Destination" 'Name of arg for the movie destination folder. 
Const ARG_JOBS_DEST_FOLDER = "Job" 'Name of arg for the destination of the created ".jobs" file. 
Const ARG_RECURSE_FOLDERS = "Recurse" 'Name of arg that defines whether you want to recursively search for source movies.

'Normal global declarations
Dim gintJobCounter, gblnRecurse
Main

Sub Main
Dim strVcfFile, strVcfData
Dim strInputStartingDirectory, strOutputDirectory, strJobsFolder, strJobsFile
Dim strMessage, strExtensions, strExtension
Dim fs, fil, fils, fols, fol
Dim dtCreated
	'First see if help was requested
	If WScript.Arguments.Named.Exists("?") Then
		ShowHelp ""
		Exit Sub
	End If
	Set fs = CreateObject("Scripting.FileSystemObject")
	'Get the VCF data
	If Wscript.Arguments.Named.Exists(ARG_VCF_FILE) Then
		strVcfFile = Wscript.Arguments.Named.Item(ARG_VCF_FILE)
	Else
		strVcfFile = InputBox("Enter the full path to the VirtualDub ""VCF"" configuration file to be used:" & vbCrLf & vbCrLf & "(Try shift-right-clicking on the desired file, select ""Copy as path"", then paste the result here)","Select VCF File", "")
	End If
	If strVcfFile = "" Then Exit Sub 'User hit Cancel on file selection dialog
	strVcfFile = Replace(strVcfFile, """", "") 
	If Not fs.FileExists(strVcfFile) Then
		 ShowHelp "File error: " & strVcfFile
		 Exit Sub
	End If
	If Lcase(Right(strVcfFile, 4)) <> ".vcf" Then
		ShowHelp "A VCF file has to have a VCF file extension! The file you chose doesn't."
		Exit Sub
	End If
	strVcfData = File2String(strVcfFile)
	If InStr(strVcfData, "VirtualDub.") = 0 Then
		ShowHelp "Not a valid VCF file!"
		Exit Sub
	End If
	'Process what kind of files?
	If Wscript.Arguments.Named.Exists(ARG_MOVIE_SOURCE_EXTENSIONS) Then
		strExtensions = Wscript.Arguments.Named.Item(ARG_MOVIE_SOURCE_EXTENSIONS)
	Else
		strExtensions = InputBox("Process what types of files (comma-delimited list)?",,"avi, mpg, mpeg")
	End If
	'Get the input directory
	If Wscript.Arguments.Named.Exists(ARG_MOVIE_SOURCE_FOLDER) Then
		strInputStartingDirectory = Wscript.Arguments.Named.Item(ARG_MOVIE_SOURCE_FOLDER)
	Else
		strInputStartingDirectory = fs.GetParentFolderName(strVcfFile)
		strInputStartingDirectory = InputBox("Source / starting directory for movies:" & vbCrLf & vbCrLf & "(Try shift-right-clicking on the desired folder, select ""Copy as path"", then paste the result here)", "Source Folder", strInputStartingDirectory)
		strInputStartingDirectory = Replace(strInputStartingDirectory, """", "") 
		If strInputStartingDirectory = "" Then Exit Sub
	End If
	If Not fs.FolderExists(strInputStartingDirectory) Then
		ShowHelp "Selected directory" & vbCrLf & strInputStartingDirectory & vbCrLf & "does not exist"
		Exit Sub
	End If
	'Should we recurse?
	If Wscript.Arguments.Named.Exists(ARG_RECURSE_FOLDERS) Then
		gblnRecurse = Wscript.Arguments.Named.Item(ARG_RECURSE_FOLDERS)
		If LCase(gblnRecurse) = "true" Then
			gblnRecurse = True
		Else
			gblnRecurse = False
		End If
	Else
		If MsgBox("Should I ""recurse"" (search subdirectories looking for all source movies)?",vbYesNo, "Recurse?") = vbYes Then
			gblnRecurse = True
		Else
			gblnRecurse = False
		End If
	End If
	'Get the output directory
	If Wscript.Arguments.Named.Exists(ARG_MOVIE_DEST_FOLDER) Then
		strOutputDirectory = Wscript.Arguments.Named.Item(ARG_MOVIE_DEST_FOLDER)
	Else
		strOutputDirectory = InputBox("Enter one of either:" & vbCrLf & "  * The full path to the destination folder for the modified videos" & vbCrLf & "  * The bare name of a new destination folder to be created in each source directory" & vbCrLf & "  * Nothing (to create new folders named after the script)" ,"Destination Folder","")
		If strOutputDirectory = "" Then strOutputDirectory = fs.GetBaseName(WScript.ScriptName)
	End If
	If InStr(strOutputDirectory, "\") <> 0 Then
		If Not fs.FolderExists(strOutputDirectory) Then
			ShowHelp "Destination directory" & vbCrLf & strOutputDirectory & vbCrLf & "does not exist"
			Exit Sub
		End If
		If strOutputDirectory = strInputStartingDirectory Then
			ShowHelp "Destination directory can't be the same as source directory because the output files will have the same names as the input files!"
			Exit Sub
		End If
	End If
	'Get the JOBS folder destination
	If Wscript.Arguments.Named.Exists(ARG_JOBS_DEST_FOLDER) Then
		strJobsFolder = Wscript.Arguments.Named.Item(ARG_JOBS_DEST_FOLDER)
	Else
		strJobsFolder = InputBox("Enter the location where the new job file """ & fs.GetBaseName(WScript.ScriptName) & ".jobs"" should be placed:" & vbCrLf & vbCrLf & "(Try shift-right-clicking on the desired folder, select ""Copy as path"", then paste the result here)", "Jobs file destination", fs.GetParentFolderName(WScript.ScriptFullName))
	End If
	strJobsFolder = Replace(strJobsFolder, """", "")
	If strJobsFolder = "" Then
		strJobsFolder = fs.GetParentFolderName(WScript.ScriptFullName)
	End If
	If Not fs.FolderExists(strJobsFolder) Then
		ShowHelp "The specified folder for the "".jobs"" folder doesn't exist."
		Exit Sub
	End If
	'Now that we have the JOBS folder, go ahead and generate the actual name we'll use
	strJobsFile = fs.BuildPath(strJobsFolder, fs.GetBaseName(WScript.ScriptName) & ".jobs")
	If fs.FileExists(strJobsFile) Then
		If MsgBox("That jobs file already exists! Press ""OK"" to delete it (and replace it with a new file) or ""Cancel"" to quit.",vbOkCancel, "Replace?") = vbOk Then
			fs.DeleteFile strJobsFile
		Else
			Exit Sub
		End If
	End If
	'Process all AVI files
	gintJobCounter = 0
	RecurseFiles strInputStartingDirectory, strExtensions, strOutputDirectory, strVcfData, strJobsFile
	'Start the job file with the final job count
	String2File "//" & vbCrLf & "// $numjobs " & gintJobCounter & vbCrLf & "//" & vbCrLf & File2String(strJobsFile), strJobsFile
	'Display the "finished" message
	MsgBox "Job file """ & FileNameLikeMine("jobs") & """ created." & vbCrLf & "Reminder: If the ""jobs"" file isn't in the VirtualDub program folder, you'll need to move it. If VirtualDub is running, you'll need to restart it before it will see your new ""jobs"" file."
End Sub

Sub RecurseFiles(strFolder, strExtensions, strOutputDirectory, strVcfData, strJobsFile)
Dim fs, fils, fil, fols, fol, objFolder
Dim strExtension
	'On Error Resume Next
	Set fs = CreateObject("Scripting.FileSystemObject")
	Set objFolder = fs.GetFolder(strFolder)
	'Get each file in turn
	Set fils = objFolder.Files
	If Err.Number <> 0 Then Exit Sub
	For Each fil In fils
		'Insert code here to process file "fil"
		For Each strExtension In Split(strExtensions, ",")
			If Lcase(Mid(fil.Name, InStrRev(fil.Name, ".") + 1)) = Lcase(Trim(strExtension)) Then 
				AddJob fil.Name, strFolder, strOutputDirectory, strVcfData, strJobsFile
				gintJobCounter = gintJobCounter + 1
				Exit For
			End If
		Next
	Next
	'Don't recursively search subfolders unless we were specifically told to
	If gblnRecurse Then
		'Check for any sub folders and recursively process them
		Set fols = objFolder.SubFolders
		For each fol in fols
			If Lcase(fol.Name) <> "recycled" Then
				RecurseFiles fol.Path, strExtensions, strOutputDirectory, strVcfData, strJobsFile
			End If
		Next
	End If
End Sub

Sub AddJob(strFile, strInDir, strOutDir, strVcf, strJobsFile)
'Adds a job to the jobs file
Dim fs, ts, strActualOutDir
Const ForAppending = 8
	Set fs = CreateObject("Scripting.FileSystemObject")
	'If the movie output folder was defined, use it. Otherwise create a new folder to store movies.
	If InStr(strOutDir, "\") <> 0 Then
		strActualOutDir = strOutDir
	Else
		strActualOutDir = fs.BuildPath(strInDir, strOutDir)
		If Not fs.FolderExists(strActualOutDir) Then fs.CreateFolder strActualOutDir
	End If
	'Add the job data for this particular movie file
	Set ts = fs.OpenTextFile(strJobsFile, ForAppending, True)
	ts.WriteLine "// $job """ & strFile & """"
	ts.Write "// $input """
	ts.Write fs.BuildPath(strInDir, strFile)
	ts.WriteLine """"
	ts.Write "// $output """
	ts.Write fs.BuildPath(strActualOutDir, strFile)
	ts.WriteLine """"
	ts.WriteLine "// $state 0"
	ts.WriteLine "// $start_time 0 0"
	ts.WriteLine "// $end_time 0 0"
	ts.WriteLine "// $script"
	ts.WriteLine ""
	ts.Write "VirtualDub.Open("""
	ts.Write Replace(fs.BuildPath(strInDir, strFile), "\", "\\")
	ts.WriteLine """,0,0);"
	ts.WriteLine strVcf
	ts.Write "VirtualDub.SaveAVI("""
	ts.Write Replace(fs.BuildPath(strActualOutDir, strFile), "\", "\\")
	ts.WriteLine """);"
	ts.WriteLine "VirtualDub.Close();"
	ts.WriteLine ""
	ts.WriteLine "// $endjob"
	ts.WriteLine "//"
	ts.WriteLine "//--------------------------------------------------"
	ts.Close
End Sub

Sub ShowHelp(strError)
Dim strMessage
	strMessage = strError
	If strMessage <> "" Then strMessage = strMessage & vbCrLf
	strMessage = strMessage & vbCrLf & WScript.ScriptName
	strMessage = strMessage & vbCrLf & "Creates a VirtualDub "".jobs"" job for multiple movie files based on settings "
	strMessage = strMessage & vbCrLf & "from a single movie's "".vcf"" settings configuration file. The ""/" & ARG_MOVIE_DEST_FOLDER & ":"" "
	strMessage = strMessage & vbCrLf & "argument can be a full path, a bare folder name, or can be empty to use the "
	strMessage = strMessage & vbCrLf & "script's name to create new folders. The ""/" & ARG_JOBS_DEST_FOLDER & ":"" argument can be a full "
	strMessage = strMessage & vbCrLf & "folder path or can be empty to default to the script's directory. Any missing "
	strMessage = strMessage & vbCrLf & "arguments will be prompted for. Command-line example:"
	strMessage = strMessage & vbCrLf & vbCrLf & WScript.ScriptName & " /" & ARG_VCF_FILE & ":""C:\file.vcf"" /" & ARG_MOVIE_SOURCE_EXTENSIONS & ":""avi,mpg"" /" & ARG_MOVIE_SOURCE_FOLDER & ":""C:\OldMovies"" /" & ARG_MOVIE_DEST_FOLDER & ":""NewMovies"" /" & ARG_JOBS_DEST_FOLDER & ":"""" /" & ARG_RECURSE_FOLDERS & ":True"
	WScript.Echo strMessage
End Sub

Function File2String(strFile) 'As String
'Reads a file and returns the contents as a string
Dim fs 'As Scripting.FileSystemObject
Dim ts 'As Scripting.TextStream
Const ForReading = 1
	Set fs = CreateObject("Scripting.FileSystemObject")
	If fs.FileExists(strFile) Then
		Set ts = fs.OpenTextFile(strFile, ForReading, True)
		If ts.AtEndOfStream Then
			File2String =""
		Else
			File2String = ts.ReadAll
		End If
		ts.Close
	Else
		File2String = ""
	End If
End Function

Sub String2File(strData, strFileName)
'Writes a string to a file
Dim fs 'As Scripting.FileSystemObject
Dim ts 'As Scripting.TextStream
Const ForWriting = 2
	Set fs = CreateObject("Scripting.FileSystemObject")
	Set ts = fs.OpenTextFile(strFileName, ForWriting, True)
	ts.Write(strData)
	ts.Close
End Sub

Sub AppendLineToFile(strText, strFile)
Dim fs 'As Scripting.FileSystemObject
Dim ts 'As Scripting.TextStream
Const ForAppending = 8
	Set fs = CreateObject("Scripting.FileSystemObject")
	Set ts = fs.OpenTextFile(strFile, ForAppending, True)
	ts.WriteLine strText
	ts.Close
End Sub

Function FileNameLikeMine(strFileExtension) 'As String
'Returns a file name the same as the script name except
'for the file extension. 
Dim fs 'As Object
Dim strExtension 'As String
	Set fs = CreateObject("Scripting.FileSystemObject")
	strExtension = strFileExtension
	If Len(strExtension) < 1 Then strExtension = "txt"
	If strExtension = "." Then strExtension = "txt"
	If Left(strExtension,1) = "." Then strExtension = Mid(strExtension, 2)
	FileNameLikeMine = Left(Wscript.ScriptFullName, InstrRev(Wscript.ScriptFullName, ".")) & strExtension
End Function
